# Automation Scripts Guide for Burst Statistics Plugin

This README serves as a comprehensive guide for using the automation scripts located in the `/automation/` directory of the Burst Statistics Plugin repository.

## Table of Contents

- [Introduction](#introduction)
- [Prerequisites](#prerequisites)
- [SSH Key Setup](#ssh-key-setup)
- [Setting Permissions](#setting-permissions)
- [Using create_rc.sh](#using-create_rcsh)
- [Troubleshooting](#troubleshooting)

## Introduction

The automation scripts are designed to simplify the workflow for managing release candidates, uploading, and downloading translation files. This guide provides step-by-step instructions for setting up and using these scripts.

## Prerequisites

Before using these scripts, ensure you have the following installed:

1. `bash` - The Bourne Again Shell
2. `wp-cli` - The WordPress Command Line Interface, required for WordPress-related operations.
3. `rsync` and `zip` - Required for file synchronization and compression tasks.

## SSH Key Setup

1. **Open Terminal**: Launch your terminal application.
2. **Generate SSH Key**: Execute the following command, replacing `youremail@example.com` with your actual email address.
   ```bash
   ssh-keygen -t rsa -b 4096 -C "youremail@example.com"
   ```
3. **Save Key**: Press enter three times to save the key in the default location and set no passphrase.
4. **Copy SSH Key**: Run the following command to copy the SSH key to your clipboard. Make sure to include your email and `ssh-rsa`.
   ```bash
   cat ~/.ssh/id_rsa.pub
   ```
5. **Create SFTP User and Add SSH Key**: Before adding the copied SSH key, you need to create an SFTP user on your server. Once the user is created, you can add the SSH key to the user.

## Setting Permissions

To execute these scripts, you'll need to set the appropriate permissions. Navigate to the `/automation/` directory in your terminal and run:

```bash
chmod +x create_rc.sh
chmod +x upload_languages.sh
chmod +x upload_plugin.sh
```

## Using `create_rc.sh`

This script automates the process of creating a release candidate for the Burst Pro.

**How to Run**:

Navigate to the `/automation/` directory and execute:

```bash
./create_rc.sh yourUsername [--enable_upload=plugin|translation]
```

**Options**:

- `--enable_upload=plugin`: This option will enable the script to upload the plugin to `translate.really-simple-plugins.com`.

- `--enable_upload=translation`: This option will enable the script to upload the translation files to `translate.really-simple-plugins.com`.

- Leaving `--enable_upload` empty or not including it will disable the upload feature.

## Troubleshooting

If you encounter any issues while using these scripts, please check the following:

1. Ensure all prerequisites are installed.
2. Verify that the SSH keys are correctly set up.
3. Make sure you have the necessary permissions to execute the scripts.
